# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package DBOps;

use strict;
use warnings FATAL => 'all';
use diagnostics;

require "db.pl";


# 0: nick
# r: nick id
sub add_nick
{
	my ($nick) = @_;
	my $nick_id;
	my $query;

	$nick = &DB::safe_string($nick);

	&DB::transaction_start();
	$nick_id = &DB::get_sequence("nicks_id_seq");
	$query = "INSERT INTO nicks (id, nick) VALUES ($nick_id, '$nick')";
	&DB::update($query);
	&DB::transaction_end();

	return $nick_id;
}


# 0: nick
# 1: audo add
# r: nick id
sub get_nick_id
{
	my ($nick, $autoadd) = @_;
	my ($query, $result);

	$nick = &DB::safe_string($nick);

	$query = "SELECT id FROM nicks WHERE nick ILIKE '$nick'";
	$result = &DB::query($query);
	if ($result->ntuples == 1) {
		my ($nick_id) = $result->fetchrow;
		return $nick_id;
	} elsif ($result->ntuples == 0) {
		if (defined($autoadd) && $autoadd == 1) {
			return &add_nick($nick);
		}
		return;
	} else {
		return;
	}
}


# 0: chan
# r: chan id
sub add_chan
{
	my ($chan) = @_;
	my $chan_id;
	my $query;

	$chan = &DB::safe_string($chan);

	&DB::transaction_start();
	$chan_id = &DB::get_sequence("chans_id_seq");
	$query = "INSERT INTO chans (id, chan) VALUES " .
			"($chan_id, '$chan')";
	&DB::update($query);
	&DB::transaction_end();

	return $chan_id;
}


# 0: chan
# 1: autoadd
# r: chan id
sub get_chan_id
{
	my ($chan, $autoadd) = @_;
	my ($query, $result);

	$chan = &DB::safe_string($chan);

	$query = "SELECT id FROM chans WHERE chan ILIKE '$chan'";
	$result = &DB::query($query);
	if ($result->ntuples == 1) {
		my ($chan_id) = $result->fetchrow;
		return $chan_id;
	} elsif ($result->ntuples == 0) {
		if (defined($autoadd) && $autoadd == 1) {
			return &add_chan($chan);
		}
		return;
	} else {
		return;
	}
}


# 0: nick
# 1: chan
sub add_scoreboard
{
	my ($nick, $chan, $pos) = @_;
	my $query;

	$nick = &DB::safe_string($nick);
	$chan = &DB::safe_string($chan);

	$query = "INSERT INTO scoreboard (nick, chan) " .
			"VALUES ($nick, $chan)";
	&DB::update($query);
}


# 0: nick
# 1: chan
# 2: auto-add
# r: (fame, shame, cheat, wanha)
sub get_scoreboard
{
	my ($nick, $chan, $autoadd) = @_;
	my ($query, $result);

	$nick = &DB::safe_string($nick);
	$chan = &DB::safe_string($chan);

	$query = "SELECT fame, shame, cheat, wanha FROM scoreboard " .
			"WHERE nick = $nick AND chan = $chan";
	$result = &DB::query($query);
	if ($result->ntuples == 1) {
		my ($fame, $shame, $cheat, $wanha) = $result->fetchrow;
		return ($fame, $shame, $cheat, $wanha);
	} elsif ($result->ntuples == 0) {
		if (defined($autoadd) && $autoadd == 1) {
			&add_scoreboard($nick, $chan);
			return (0, 0, 0, 0);
		} else {
			return;
		}
	} else {
		return;
	}
}


# 0: id
# ret: nick
sub get_nick_name
{
	my ($id) = @_;
	my $query;
	my $result;
	
	$id = &DB::safe_string($id);

	$query = "SELECT nick FROM nicks WHERE id = $id";
	$result = &DB::query($query);
	($result->ntuples == 0) && return;

	my ($nick) = $result->fetchrow;
	return $nick;
}


# ...: nick id
# %r: id->nick
sub get_nick_names
{
	my @ids = @_;
	my ($query, $result);

	$query = "SELECT id, nick FROM nicks WHERE ";
	foreach (@ids) {
		$_ = &DB::safe_string($_);
		$query .= "id = $_ OR "
	}
	$query =~ s/OR $//;

	$result = &DB::query($query);
	my %nick;
	while (my ($id, $nick) = $result->fetchrow) {
		$nick{$id} = $nick;
	}

	return %nick;
}


# 0: id
# ret: chan
sub get_chan_name
{
	my ($id) = @_;
	my $query;
	my $result;

	$id = &DB::safe_string($id);

	$query = "SELECT chan FROM chans WHERE id = $id";
	$result = &DB::query($query);
	($result->ntuples == 0) && return;

	my ($chan) = $result->fetchrow;
	return $chan;
}


# 0: url
# 1: chan id
# 2: nick id
# 3: time = localtime()
sub add_url
{
	my ($url, $chan, $nick, $time) = @_;
	my $query;

	$url = &DB::safe_string($url);
	$chan = &DB::safe_string($chan);
	$nick = &DB::safe_string($nick);

	if (defined($time)) {
		$time = &DB::safe_string($time);
		$query = "INSERT INTO urls (url, nick, chan, time) VALUES " .
			"('$url', $nick, $chan, '$time')";
	} else {
		$query = "INSERT INTO urls (url, nick, chan) VALUES " .
			"('$url', $nick, $chan)";
	}
	
	&DB::update($query);
}


# 0: url
# 1: chan
# r: (id, count, time, nick)
sub get_url_info_by_url
{
	my ($url, $chan) = @_;
	my ($query, $result);

	$url = &DB::safe_string($url);
	$chan = &DB::safe_string($chan);

	$query = "SELECT id, count, " . &get_date_fmt("time") . ", nick " .
			"FROM urls " .
			"WHERE url = '$url' AND chan = $chan";
	$result = &DB::query($query);
	($result->ntuples == 0) && return;

	my ($id, $count, $time, $nick) = $result->fetchrow;
	return ($id, $count, $time, $nick);
}


# 0: url id
# r: (count, time, nick, url)
sub get_url_info_by_id
{
	my ($url_id) = @_;
	my ($query, $result);

	$url_id = &DB::safe_string($url_id);

	$query = "SELECT count, " . &get_date_fmt("time") . ", nick, url " .
			"FROM urls WHERE id = $url_id";
	$result = &DB::query($query);
	($result->ntuples == 0) && return;

	my ($count, $time, $nick, $url) = $result->fetchrow;
	return ($count, $time, $nick, $url);
}


# 0: field
# 1: as = field
# r: to_char for date
sub get_date_fmt
{
	my ($field, $as) = @_;

	$field = &DB::safe_string($field);

	if (! defined($as)) {
		($as) = $field =~ /\.(.*)$/;
		$as = $field if (! defined($as));
	} else {
		$as = &DB::safe_string($as);
	}

	return "TO_CHAR($field, 'Mon dd hh24:mi:ss yyyy') AS $as";
}


return 1;
